#include "midiRecorder.h"
#include "ofxMidi.h"

#include <set>
#include <vector>
#include <math.h>
#include <string>
#include <sstream>
using namespace std;

//--------------------------------------------------------------
void MidiRecorder::setup(){
	
	_midiIn.listPorts();
	_midiIn.openPort(1);
	_midiIn.addListener(this);
	
	ofSetVerticalSync(true);
	ofSetFrameRate(30);
	
	//testRecorder();
	
	_franklin_book.loadFont("frabk.ttf", 24);
	
	_master_time = 0;
	
	_width = _height = 1000;
	ofSetWindowShape(_width, _height);
	
	_loopSeconds = 60*2;
	_showTime = true;
	
	
	addPoints();
	
}


//--------------------------------------------------------------
void MidiRecorder::addPoints(){
	int numCircles = 200;
	int numGroups = 5;
	
	vector<float> groupSpeed, groupSpeedMax;
	
	// TODO fill in w/ musical ratios
	groupSpeed.push_back(3);
	groupSpeedMax.push_back(5);
	
	groupSpeed.push_back(-6);
	groupSpeedMax.push_back(-3);
	
	groupSpeed.push_back(-3);
	groupSpeedMax.push_back(-1.5);
	
	groupSpeed.push_back(2);
	groupSpeedMax.push_back(3);
	
	groupSpeed.push_back(2);
	groupSpeedMax.push_back(4);
	
	for(int g=0; g<numGroups; ++g) {
		
		ofColor drawColor;
		drawColor.r = ofRandom(0, 255);
		drawColor.g = ofRandom(0, 255);
		drawColor.b = ofRandom(0, 255);
		drawColor.a = 20;
		
		int circleRange = numCircles/numGroups;
		for(int c=0; c<circleRange; ++c) {
			
			// actually repulsion radius
			float drawRadius			= 1;
			
			float rotationRadius	= ofMap(c, 0, circleRange, 0, min(ofGetWidth()/2, ofGetHeight()/2)); 
			
			ofPoint origin(_width/2, _height/2);
			
			float rotationSpeed		= ofMap(c, 0, circleRange, groupSpeed[g], groupSpeedMax[g]); 
			
			float RRModulation		= .2;
			
			float angleOffset			= 2 * M_PI * g / numGroups;
			
			_points.push_back(WhitneyPoint(drawRadius,
																		rotationRadius,
																		origin,
																		rotationSpeed,
																		RRModulation,
																		drawColor,
																		angleOffset));
		}		
	}
}


//--------------------------------------------------------------
void MidiRecorder::update(){
	_master_time = fmod(ofGetElapsedTimef(), _loopSeconds);

	vector<WhitneyPoint>::iterator itPoint;
	
	vector<IdValue> values = _recorder.getValues(_master_time);
	
	vector<IdValue>::iterator itVal = values.begin();
	for(itVal; itVal != values.end(); ++itVal) {
		//ret << it->id << "/" << it->value << "\n";

		float pct = itVal->value / 127.0f;
		
		switch(itVal->id) {
			case 108:
				for(itPoint = _points.begin(); itPoint != _points.end(); ++itPoint) {
					itPoint->setRotationRadiusPct(pct);
				}
				break;
				
			case 109:
				for(itPoint = _points.begin(); itPoint != _points.end(); ++itPoint) {
					itPoint->setDrawRadiusPct(pct * 30);
				}
				break;
				
			case 110:
				for(itPoint = _points.begin(); itPoint != _points.end(); ++itPoint) {
					itPoint->setShake(pct * 20);
				}
				break;
				
			case 111:
				for(itPoint = _points.begin(); itPoint != _points.end(); ++itPoint) {
					itPoint->setRotationSpeedPct(ofMap(pct, 0, 1, -3, 3, true));
				}
				break;	

			case 114:
				for(itPoint = _points.begin(); itPoint != _points.end(); ++itPoint) {
					itPoint->setAlpha(pct);
				}
				break;	
				
			default:
				break;
		}
	}
	
	itPoint = _points.begin();
	for(itPoint; itPoint != _points.end(); ++itPoint) {
		itPoint->update();
	}
}

//--------------------------------------------------------------
void MidiRecorder::draw(){
	
	ofBackground(0, 0, 0);
	
	ofEnableAlphaBlending();
	

	vector<WhitneyPoint>::iterator it = _points.begin();
	for(it; it!=_points.end(); ++it) {
		it->draw();
	}
	
	char frameRate[10];
	sprintf(frameRate, "%2.1f", ofGetFrameRate());
	
	char clockString[255];
	sprintf(clockString, "%3.1f/%3.1d", _master_time, _loopSeconds);
	
	if(_showTime) {
		ofSetColor(125, 125, 125, 125);
		ofRect(25, 25, 300, 100);
	
		ofSetColor(255, 255, 255, 255);
		_franklin_book.drawString(clockString, 50, 40 + _franklin_book.getLineHeight()/2);

		_franklin_book.drawString(frameRate, 50, 100);
		_franklin_book.drawString(getValues(_master_time), 50, 150);
	}
}

//--------------------------------------------------------------
void MidiRecorder::keyPressed(int key){
	
	if(key == 't') {
		_showTime = !_showTime;
	}
}

//--------------------------------------------------------------
void MidiRecorder::keyReleased(int key){

}

//--------------------------------------------------------------
void MidiRecorder::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void MidiRecorder::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void MidiRecorder::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void MidiRecorder::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void MidiRecorder::windowResized(int w, int h){

}




//--------------------------------------------------------------
void MidiRecorder::testRecorder() {
	// current
	cout << "before manipulation." << endl;
	printExistingIds();
	
	_recorder.setEvent(0, 1, 1);
	_recorder.setEvent(0, 2, 10);
	
	_recorder.setEvent(1, 1, 1);
	
	cout << "0, 1 sould exist." << endl;
	printExistingIds();
	
	_recorder.setEvent(2, 1, 1);
	_recorder.setEvent(2, 2, 2);
	_recorder.setEvent(2, 3, 3);
	
	vector<IdValue> values = _recorder.getValues(0);
	cout << "should be 0|1, 1|1, 2|1" << endl;
	printIdValues(values);

	values = _recorder.getValues(1);
	cout << "should be 0|1, 1|1, 2|1" << endl;
	printIdValues(values);
	cout << endl << endl;
	
	values = _recorder.getValues(1.5);
	cout << "should be 0|5, 1|1, 2|1.5" << endl;
	printIdValues(values);
	cout << endl << endl;

	values = _recorder.getValues(2);
	cout << "should be 0|10, 1|1, 2|2" << endl;
	printIdValues(values);
	cout << endl << endl;
	
	values = _recorder.getValues(3);
	cout << "should be 0|10, 1|1, 2|3" << endl;
	printIdValues(values);
	cout << endl << endl;
	
	values = _recorder.getValues(4);
	cout << "should be 0|10, 1|1, 2|3" << endl;
	printIdValues(values);
	cout << endl << endl;
	
}

void MidiRecorder::printIdValues(const vector<IdValue>& values) {
	vector<IdValue>::const_iterator it;
	cout << "id,values are: ";
	for(it = values.begin(); it != values.end(); ++it) {
		cout << it->id << "|" << it->value << ", ";
	}
	cout << endl;
}

void MidiRecorder::printExistingIds() {
	cout << "existing ids: ";
	cout << returnExistingIdsStr();
}

string MidiRecorder::returnExistingIdsStr() {
	const set<int>& curIds = _recorder.getExistingIds();
	stringstream ss;
	set<int>::iterator it;
	for(it = curIds.begin(); it!=curIds.end(); ++it) {
		ss << *it << ", ";
	}
	ss << endl;
	
	return ss.str();
}

string MidiRecorder::getValues(float time) {
	stringstream ret;
	
	vector<IdValue> values = _recorder.getValues(time);
	vector<IdValue>::iterator it;
	for(it = values.begin(); it != values.end(); ++it) {
		ret << it->id << "/" << it->value << "\n";
	}
	
	return ret.str();
}

void MidiRecorder::newMidiMessage(ofxMidiEventArgs& eventArgs)
{
	cout << "byteOne = " << eventArgs.byteOne << endl;
	cout << "byteTwo = " << eventArgs.byteTwo << endl;
	
	// class is thread safe
	_recorder.setEvent(eventArgs.byteOne, _master_time, eventArgs.byteTwo);
}


