/*
 *  timeEventRecorder.cpp
 *  midiRecorder
 *
 *  Created by Lucas Vickers on 9/25/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "timeEventRecorder.h"

#include <iostream>
using namespace std;


TimeEventRecorder::TimeEventRecorder() :
_map(),
_mutex()
{
	if(pthread_mutex_init(&_mutex, NULL)) {
		cerr << "ERROR init mutex." << endl;
	}
}
	

void TimeEventRecorder::setEvent(int id, float time, float value)
{
	pthread_mutex_lock(&_mutex);

	TimeEventSet& timeSet = _map[id];
	
	TimeEvent te(time, value);
	
	
	if(timeSet.find(te) != timeSet.end()) {
		timeSet.erase(timeSet.find(te));	// remove double lookup
	}
	
	timeSet.insert(TimeEvent(time, value));
	
	pthread_mutex_unlock(&_mutex);
}
	
std::vector<IdValue> TimeEventRecorder::getValues(float time)
{
	
	pthread_mutex_lock(&_mutex);
	
	std::vector<IdValue> ret;
	
	EventIdMap::iterator it;
	for(it = _map.begin(); it != _map.end(); ++it) {
		int targetId = it->first;
	
		TimeEventSet& set = it->second;
		TimeEventSet::iterator te_it, guess;
		
		te_it = set.find(TimeEvent(time, 0));
		// we hit the exact time, that's rare
		if(te_it != set.end()) {
			ret.push_back(IdValue(targetId, te_it->value));
			continue;
		}
		
		// get key that is greater or equal
		guess = set.lower_bound(TimeEvent(time, 0));
		
		if(TER_DEBUG) {
			cout << "guess is t " << guess->time << " v " << guess->value << endl;
		}
		
		// first case, we hit the mark, or we were before any data exists
		if(guess->time == time || guess == set.begin()) {
			ret.push_back(IdValue(targetId, guess->value));
			continue;
		}
		
		// we were after any data exists
		if(guess == set.end()) {
			--guess;
			ret.push_back(IdValue(targetId, guess->value));
			continue;
		}
		
		// we need to interpolate
			
		float t2 = guess->time;
		float val2 = guess->value;
		
		--guess; // move to the previous
		float t1 = guess->time;
		float val1 = guess->value;
		
		float pct = (time-t1)/(t2-t1);
		
		float interpolated_val = val1 * (1-pct) + val2 * pct;
		
		ret.push_back(IdValue(targetId, interpolated_val));
	}

	pthread_mutex_unlock(&_mutex);
	return ret;
}
		
std::set<int> TimeEventRecorder::getExistingIds()
{
	pthread_mutex_lock(&_mutex);
	
	std::set<int> ret;
	EventIdMap::iterator it;
	for(it = _map.begin(); it != _map.end(); ++it) {
		ret.insert(it->first);
	}
	
	pthread_mutex_unlock(&_mutex);
	return ret;
}
