/*
 *  point.cpp
 *  johnWhitney
 *
 *  Created by Lucas Vickers on 9/20/10.
 *  Copyright 2010 None. All rights reserved.
 *
 */

#include "whitneyPoint.h"
WhitneyPoint::WhitneyPoint(float drawRadius, 
													 float rotationRadius, 
													 ofPoint origin,
													 float rotationSpeed,
													 float RRModulation,
													 ofColor color,
													 float angleOffset) : 
drawRadius(drawRadius),
drawRadiusPct(1),
rotationRadius(rotationRadius),
rotationRadiusPct(1),
origin(origin),
currentOrigin(origin),
rotationSpeed(rotationSpeed),
rotationSpeedPct(1),
RRModulation(RRModulation),
color(color),
angleOffset(angleOffset),
curColor(color)
{
	lastUpdateTime = ofGetElapsedTimef();
	numParticles = 50;
	
	
	for (int i = 0; i < numParticles; i++){
		particle* p = new particle();
		myParticles.push_back(p);
		myParticles[i]->pos.set(ofRandom(-10, 10), ofRandom(-10,10));
		myParticles[i]->vel.set(0,0);
	}
	
}

void WhitneyPoint::draw() 
{
	int recMode = ofGetRectMode();
	ofSetRectMode(OF_RECTMODE_CENTER);	
	
	
	ofSetColor(curColor.r, curColor.g, curColor.b, curColor.a);
	
	vector<particle*>::iterator it;
	
	ofPushMatrix(); 
	{
		ofTranslate(curLocation.x, curLocation.y);
		for(it = myParticles.begin(); it != myParticles.end(); ++it) {
			(*it)->draw();
		}
	}
	ofPopMatrix();
	
	
	
	//ofSetColor(curColor.r, curColor.g, curColor.b, curColor.a);
	//ofFill();
	//ofCircle(curLocation.x, curLocation.y, drawRadius * drawRadiusPct);
	
	ofSetRectMode(recMode);
}


// comparison routine for sort...
bool comparisonFunction(  particle * a, particle * b ) { 
	return a->pos.x < b->pos.x; 
}  

void WhitneyPoint::update()
{
	float curTime = ofGetElapsedTimef();
	
	anglePosition = anglePosition + ((curTime - lastUpdateTime) * rotationSpeed * rotationSpeedPct);
	
	float radius = rotationRadius * rotationRadiusPct;
	
	curLocation.x = currentOrigin.x + radius * cos(anglePosition + angleOffset);
	curLocation.y = currentOrigin.y + radius * -sin(anglePosition + angleOffset);
	
	curColor.r = color.r;
	curColor.g = color.g;
	curColor.b = color.b;
	curColor.a = color.a;
	
	lastUpdateTime = curTime;
	
	// update shit
	
	ofBackground(0,0,0); //Grey background, NY style
	
	
	vector<particle*>::iterator it, it_in;
	for(it = myParticles.begin(); it != myParticles.end(); ++it) {
		(*it)->resetForce();
	}
	
	float repulsionRadius = drawRadius * drawRadiusPct;
	if(repulsionRadius == 0) {
		repulsionRadius = .01;
	}
	
	/*
	sort( myParticles.begin(), myParticles.end(), comparisonFunction );               // sort them!
	 
	for (int i = 0; i < myParticles.size(); i++){
		for (int j = i-1; j >= 0; j--){
			if ( fabs(myParticles[j]->pos.x - myParticles[i]->pos.x) >  repulsionRadius) break;
			
			myParticles[i]->addRepulsionForce(*myParticles[j], repulsionRadius, 1.5f);
		}
	}
	 */
	
	for(it = myParticles.begin(); it != myParticles.end(); ++it) {
		(*it)->resetForce();
		(*it)->addAttractionForce(0, 0, ofGetWidth(), .5);
	}
	
	for(it = myParticles.begin(); it != myParticles.end(); ++it) {
		for(it_in = it; it_in != myParticles.end(); ++it_in) {
			(*it)->addRepulsionForce(*(*it_in), repulsionRadius, 1.5f);
		}
	}
	
	
	for(it = myParticles.begin(); it != myParticles.end(); ++it) {
		(*it)->addDampingForce();
		(*it)->update();
	}
	
}

void WhitneyPoint::setShake(float shake)
{
	currentOrigin.x = origin.x + ofRandom(-shake, shake);
	currentOrigin.y = origin.y + ofRandom(-shake, shake);
}

void WhitneyPoint::setDrawRadiusPct(float setDrawRadiusPct)
{
	drawRadiusPct = setDrawRadiusPct;
}

void WhitneyPoint::setRotationRadiusPct(float setRotationRadiusPct)
{
	rotationRadiusPct = setRotationRadiusPct;
}

void WhitneyPoint::setRotationSpeedPct(float setRotationSpeedPct)
{
	rotationSpeedPct = setRotationSpeedPct;
}

void WhitneyPoint::setAlpha(float alpha)
{
	color.a = 100 * alpha;
	if(color.a < 1)
		color.a = 1;
}